"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable functional/prefer-immutable-types */
const utils_1 = require("@typescript-eslint/utils");
const common_1 = require("./common");
/**
 * An ESLint rule to ban partial division.
 */
// eslint-disable-next-line total-functions/no-unsafe-readonly-mutable-assignment
const noPartialDivision = (0, common_1.createRule)({
    name: "no-partial-division",
    meta: {
        type: "problem",
        docs: {
            description: "Bans partial division.",
            recommended: "error",
        },
        messages: {
            errorStringGeneric: "Division is partial. You should wrap it in a wrapper that returns undefined when the denominator is zero.",
        },
        schema: [],
    },
    create: (context) => {
        return {
            // eslint-disable-next-line functional/no-return-void
            BinaryExpression: (node) => {
                // eslint-disable-next-line functional/no-conditional-statements
                if (node.operator !== "/") {
                    // Binary expressions other than division are safe.
                    return;
                }
                // eslint-disable-next-line functional/no-conditional-statements
                if (node.right.type === utils_1.AST_NODE_TYPES.Literal &&
                    node.right.value !== 0 &&
                    node.right.value !== 0n) {
                    // Division by a literal that isn't zero is safe.
                    return;
                }
                // TODO expand this to inspect type information and not report an issue
                // if the type of the denominator is a literal numeric type (non-zero, of course)
                // even when not a literal value.
                // All other division is not provably safe.
                // eslint-disable-next-line functional/no-expression-statements
                context.report({
                    node: node,
                    messageId: "errorStringGeneric",
                });
            },
        };
    },
    defaultOptions: [],
});
exports.default = noPartialDivision;
